import unittest
import numpy as np
from tests_configuration import *


class TestStimuliCSVFileFormat(unittest.TestCase):

    # 1) test file name format: must have the form '<dataset_name>_stimuli.csv'
    def test_csv_name_suffix(self):
        csv_name = CFG.STIMULI_PATH.split('/').pop()
        # At most one '_' in csv name, separating dataset name and stimuli
        self.assertEqual(len(csv_name.split('_')), 2)
        # must finish by '_stimuli.csv'
        self.assertEqual(csv_name.split('_').pop(), 'stimuli.csv')

    # 2) Test csv name prefix (same as the folder)
    def test_csv_name_prefix(self):
        # Extract dataset name
        dataset_name = CFG.STIMULI_PATH.split('/')[-3]
        # Extract csv name
        csv_name = CFG.STIMULI_PATH.split('/').pop()
        csv_name_prefix = csv_name.split('_')[0]
        self.assertEqual(dataset_name, csv_name_prefix)

    # 3) test keys list
    def test_key_contents(self):
        for key in CFG.STIMULI_DATA.keys():
            if key.startswith('Unnamed'):
                continue
            self.assertIn(key, ['index', '#file_source', '#file_extract',
                                'onset', 'offset', '#phone',
                                'context', 'language', 'speaker',
                                'prev_phone', 'next_phone', 'dataset'])

    # 4) test each key format

    # 4) c) test onset and offset format (offset superior to onset)
    def test_onset_and_offset_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['onset'][i], float)
            self.assertIsInstance(CFG.STIMULI_DATA['offset'][i], float)
            self.assertLess(CFG.STIMULI_DATA['onset'][i], CFG.STIMULI_DATA['offset'][i])

    # 4) e) test language format (str, capital letters)
    def test_language_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['language'][i], str)
            self.assertTrue(CFG.STIMULI_DATA['language'][i].isupper())

    # 4) f) test speaker format
    def test_speaker_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['speaker'][i], str)

    # 4) r) Test phone_X format (not in capital letter, IPA alphabet)
    def test_phone_X_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['#phone'][i], str)
            self.assertIn(CFG.STIMULI_DATA['#phone'][i], CFG.IPA)

    # 4) s) Test context format
    def test_context_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['context'][i], str)
            self.assertEqual(CFG.STIMULI_DATA['context'][i].split('_')[0], CFG.STIMULI_DATA['prev_phone'][i])
            self.assertEqual(CFG.STIMULI_DATA['context'][i].split('_')[1], CFG.STIMULI_DATA['next_phone'][i])

    # 4) t) Test prev_phone format
    def test_prev_phone_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['prev_phone'][i], str)
            self.assertIn(CFG.STIMULI_DATA['prev_phone'][i], CFG.IPA)

    # 4) u) Test next_phone format
    def test_next_phone_format(self):
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertIsInstance(CFG.STIMULI_DATA['next_phone'][i], str)
            self.assertIn(CFG.STIMULI_DATA['next_phone'][i], CFG.IPA)

    # 4) i) test dataset key format (same name as the csv prefix and folder prefix)
    def test_dataset_key_format(self):
        dataset_name = CFG.STIMULI_PATH.split('/')[-3]
        random_indexes = np.random.randint(0, CFG.NB_STIMULI, CFG.NB_TESTS)
        for i in random_indexes:
            self.assertEqual(CFG.STIMULI_DATA['dataset'][i], dataset_name)



if __name__ == '__main__':
    unittest.main()
    print('TESTS ON STMULI FINISHED')
